@@ -19,25 +19,36 @@
 
 #include "notificationmonitor.h"
 #include "notificationmodel.h"
+#include "notificationfiltermodel.h"
 #include "utils.h"
 
 #include <akonadi/private/protocol_p.h>
 #include <AkonadiWidgets/ControlGui>
 
-
+#include <QFileDialog>
+#include <QFile>
 #include <QHeaderView>
 #include <QCheckBox>
+#include <QLabel>
 #include <QMenu>
+#include <QMessageBox>
 #include <QTreeView>
 #include <QVBoxLayout>
 #include <QSplitter>
 #include <QStandardItemModel>
 #include <QStandardItem>
 #include <QItemSelectionModel>
+#include <QPushButton>
+#include <QJsonArray>
+#include <QJsonDocument>
+#include <QJsonObject>
 
+#include <KCheckComboBox>
 #include <KSharedConfig>
 #include <KConfigGroup>
 
+using KPIM::KCheckComboBox;
+
 Q_DECLARE_METATYPE(Akonadi::ChangeNotification)
 
 NotificationMonitor::NotificationMonitor(QWidget *parent) :
@@ -46,20 +57,31 @@ NotificationMonitor::NotificationMonitor(QWidget *parent) :
     m_model = new NotificationModel(this);
     m_model->setEnabled(false);   // since it can be slow, default to off
 
+    m_filterModel = new NotificationFilterModel(this);
+    m_filterModel->setSourceModel(m_model);
+
     QVBoxLayout *layout = new QVBoxLayout(this);
+    auto hLayout = new QHBoxLayout;
+    layout->addLayout(hLayout);
 
     QCheckBox *enableCB = new QCheckBox(this);
     enableCB->setText(QStringLiteral("Enable notification monitor"));
     enableCB->setChecked(m_model->isEnabled());
     connect(enableCB, &QCheckBox::toggled, m_model, &NotificationModel::setEnabled);
-    layout->addWidget(enableCB);
+    hLayout->addWidget(enableCB);
 
+    hLayout->addWidget(new QLabel(QStringLiteral("Types:"), this));
+    hLayout->addWidget(mTypeFilterCombo = new KCheckComboBox(this));
+    hLayout->addStretch();
+    mTypeFilterCombo->setSizeAdjustPolicy(QComboBox::AdjustToContents);
+    mTypeFilterCombo->setMinimumWidth(fontMetrics().width(QStringLiteral("Subscription,Items,Collections")) + 60); // make it wide enough for most use cases
+    m_filterModel->setTypeFilter(mTypeFilterCombo);
 
     m_splitter = new QSplitter(this);
     layout->addWidget(m_splitter);
 
     m_treeView = new QTreeView(this);
-    m_treeView->setModel(m_model);
+    m_treeView->setModel(m_filterModel);
     m_treeView->expandAll();
     m_treeView->setAlternatingRowColors(true);
     m_treeView->setContextMenuPolicy(Qt::CustomContextMenu);
@@ -74,6 +96,14 @@ NotificationMonitor::NotificationMonitor(QWidget *parent) :
     m_ntfView->setEditTriggers(QAbstractItemView::NoEditTriggers);
     m_splitter->addWidget(m_ntfView);
 
+    auto *h = new QHBoxLayout;
+    layout->addLayout(h);
+
+    auto btn = new QPushButton(QStringLiteral("Save to File..."));
+    connect(btn, &QPushButton::clicked, this, &NotificationMonitor::saveToFile);
+    h->addWidget(btn);
+    h->addStretch(1);
+
     onNotificationSelected({});
 
     KConfigGroup config(KSharedConfig::openConfig(), "NotificationMonitor");
@@ -102,18 +132,24 @@ void NotificationMonitor::contextMenu(const QPoint & /*pos*/)
 
 void NotificationMonitor::onNotificationSelected(const QModelIndex &index)
 {
-    auto model = qobject_cast<QStandardItemModel*>(m_ntfView->model());
     const auto state = m_ntfView->header()->saveState();
+    populateNtfModel(index);
+    m_ntfView->header()->restoreState(state);
+    m_ntfView->expandAll();
+}
+
+void NotificationMonitor::populateNtfModel(const QModelIndex &index)
+{
+    auto model = qobject_cast<QStandardItemModel*>(m_ntfView->model());
     model->clear();
     model->setHorizontalHeaderLabels({ QStringLiteral("Properties"), QStringLiteral("Values") });
-    m_ntfView->header()->restoreState(state);
 
     const auto ntf = index.data(NotificationModel::NotificationRole).value<Akonadi::ChangeNotification>();
     if (!ntf.isValid()) {
         return;
     }
 
-    appendRow(model, QStringLiteral("Timestamp"), ntf.timestamp().toString(Qt::ISODate));
+    appendRow(model, QStringLiteral("Timestamp"), ntf.timestamp().toString(Qt::ISODateWithMs));
     appendRow(model, QStringLiteral("Type"),
               index.sibling(index.row(), NotificationModel::TypeColumn).data().toString());
     appendRow(model, QStringLiteral("Listeners"),
@@ -135,8 +171,6 @@ void NotificationMonitor::onNotificationSelected(const QModelIndex &index)
         populateSubscriptionNtfTree(model, Akonadi::Protocol::cmdCast<Akonadi::Protocol::SubscriptionChangeNotification>(ntf.notification()));
         break;
     }
-
-    m_ntfView->expandAll();
 }
 
 void NotificationMonitor::populateItemNtfTree(QStandardItemModel *model, const Akonadi::Protocol::ItemChangeNotification &ntf)
@@ -508,7 +542,7 @@ void NotificationMonitor::populateSubscriptionNtfTree(QStandardItemModel *model,
     auto item = new QStandardItem(QStringLiteral("Item Fetch Scope"));
     const auto ifs = ntf.itemFetchScope();
     appendRow(item, QStringLiteral("Requested Parts"), toString(ifs.requestedParts()));
-    appendRow(item, QStringLiteral("Changed Since"), ifs.changedSince().toString(Qt::ISODate));
+    appendRow(item, QStringLiteral("Changed Since"), ifs.changedSince().toString(Qt::ISODateWithMs));
     appendRow(item, QStringLiteral("Tag Fetch Scope"), toString(ifs.tagFetchScope()));
     QString ancestorDepth;
     switch (ifs.ancestorDepth()) {
@@ -584,3 +618,42 @@ void NotificationMonitor::populateSubscriptionNtfTree(QStandardItemModel *model,
     appendRow(item, QStringLiteral("Ignore Retrieval Errors"), toString(cfs.ignoreRetrievalErrors()));
     model->appendRow(item);
 }
+
+void NotificationMonitor::saveToFile()
+{
+    const auto filename = QFileDialog::getSaveFileName(this, QStringLiteral("Save to File..."));
+    if (filename.isEmpty()) {
+        return;
+    }
+
+    QFile file(filename);
+    if (!file.open(QIODevice::WriteOnly | QIODevice::Truncate)) {
+        QMessageBox::warning(this, QStringLiteral("Error"), QStringLiteral("Failed to open file: %1").arg(file.errorString()));
+        return;
+    }
+
+    QJsonObject json;
+    auto ntfModel = qobject_cast<QStandardItemModel*>(m_ntfView->model());
+    Q_ASSERT(ntfModel);
+
+    QJsonArray rowArray;
+    // Note that the use of m_model here means we save everything, not just what's visible (in case of filtering).
+    const int rows = m_model->rowCount();
+    for (int row = 0; row < rows; ++row) {
+        QJsonObject rowObject;
+        // The Ntf model has all the data that m_model has in its columns, apart from the session
+        rowObject.insert(QStringLiteral("Session"), m_model->index(row, NotificationModel::SessionColumn).data().toString());
+
+        populateNtfModel(m_model->index(row, 0));
+        for (int r = 0, cnt = ntfModel->rowCount(); r < cnt; ++r) {
+            const auto idx0 = ntfModel->index(r, 0);
+            const auto idx1 = ntfModel->index(r, 1);
+            rowObject.insert(idx0.data().toString(), QJsonValue::fromVariant(idx1.data()));
+        }
+
+        rowArray.append(rowObject);
+    }
+    json.insert(QStringLiteral("notifications"), rowArray);
+    QJsonDocument saveDoc(json);
+    file.write(saveDoc.toJson());
+}
